/*----------------------------------------------------------------------*\
 | Routines to convert between coordinate systems for sea ice dataset	|
 |																		|
 | mapxy and mapll are translated from Fortran code supplied by NSIDC	|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <ctype.h>
#include <math.h>

#define sind(x)		sin(x*3.141592653589793/180.0)
#define cosd(x)		cos(x*3.141592653589793/180.0)

#define CDR		57.29577951
#define RE		6378.273
#define E2		0.006693883
#define E		0.081816154
#define PI		3.141592653589793
#define SLAT	70.0

static int initialized = 0;
static double t = 0.0;
static double t2 = 0.0;
static double cm = 0.0;

static void initialize (void) {
	t = tan (PI/4.0 - SLAT/(2.0*CDR))/pow((1.0 - E*sind(SLAT))/(1.0 + E*sind(SLAT)),E/2.0);
	cm = cosd (SLAT)/sqrt(1.0 - E2*(sind(SLAT)*sind(SLAT)));
	initialized = 1;
	}

void mapll (double alat, double along, double *x, double *y) {
	double sn,xlam,t1,rho;

	sn = 1.0;
	if (alat < 0.0) sn = -1.0;
	alat *= sn;
	if (alat >= 89.995) {
		*x = 0;
		*y = 0;
		return;
		}
	t1 = tan (PI/4.0 - alat/(2.0*CDR))/pow((1.0 - E*sind(alat))/(1.0 + E*sind(alat)),E/2.0);
	if (!initialized) initialize();
	rho = RE * cm * t1/t;
	*x =  rho * sn * sind (along);
	*y = -rho      * cosd (along);
	}

void mapxy (double x, double y, char hemisphere, double *alat, double *along) {
	double sn,xlam,rho,chi;

	sn = 1.0;
	xlam = -45.0;
	if (toupper(hemisphere) == 'S') {
		sn = -1.0;
		xlam = 180.0;
		}
	rho = sqrt (x * x + y * y);
	if (rho < 0.1) {
		*alat = 90.0;
		if (sn < 0.0) *alat = -90.0;
		*along = 90.0;
		return;
		}
	if (!initialized) initialize();
	chi = PI/2.0 - 2.0 * atan (t*rho/(RE*cm));
	*alat = chi +
		((E2/2.0) + (5.0*E2*E2/24.0) + (E2*E2*E2/12.0))*sin(2.0*chi) +
		((7.0*E2*E2/48.0) + (29.0*E2*E2*E2/240.0))*sin(4.0*chi) +
		(7.0*E2*E2*E2/120.0)*sin(6.0*chi);
	*alat *= sn*CDR;
	*along = xlam + sn * CDR * atan2(x,-y);
	if (*along < 0.0) *along += 360.0;
	if (*along > 360.0) *along -= 360.0;
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
