#-------------------------------------------------------------------------
# Program to visualize sea-ice concentration data
#
# Peter N. Schweitzer (U.S. Geological Survey, Reston VA 20192)
#-------------------------------------------------------------------------

set MISSING_CODE 157
set LAND_CODE 168

set hemisphere N
set image_label .w.legend.layer.label
set image_list .w.legend.layer.list
set value_list .w.legend.layer.value
set plot .w.legend.plot
set map .w.map
set map_width 304
set map_height 448

set gridline_count 0
set show_grid 0
set show_marker 0
set mark_x 0
set mark_y 0

set year_label {"78" "79" "80" "81" "82" "83" "84" "85" "86" "87" "88" "89" "90" "91" "92" }
set month_label {"Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec" }
set label_list $month_label

#------------------------------------------------------------------------

proc ice_of_color {r g b} {
	global MISSING_CODE LAND_CODE
	if {$r == $g} {
		set i [expr ($r * 100)/255]
		return $i
		} \
	else {
		if {$r == 0} {
			return $MISSING_CODE
			} \
		else {
			return $LAND_CODE
			}
		}
	}

#------------------------------------------------------------------------

set count 0

proc load_north {} {
	global label_list month_label
	global filename im tag ok count image_list map hemisphere
	global image_label dataset_name
	global mark_x mark_y map_width map_height

	stop_cycling

	set map_width 304
	set map_height 448
	$map configure -width $map_width -height $map_height

	set hemisphere N
	set_pole N
	set dataset_name "Arctic Ocean Monthly Average\nSea-Ice Concentration"
	$image_label configure -text "$dataset_name\nPick a location on the map"

	foreach d {data/n ../data/n n} {
		if {[file exists $d] && [file isdirectory $d]} {
			break
			} \
		else {
			set d {}
			}
		}

	set label_list $month_label
	$image_list delete 0 end
	set i 0
	foreach month $label_list {
		$image_list insert end "$month"

		incr i
		set name [expr $i / 10]
		append name [expr $i % 10]
		append name n.gif
		set filename($month) $name
		# puts stdout $filename($month)

		set im($month) [image create photo -format gif -file $d/$filename($month)]
		set tag($month) [$map create image 0 0 -image $im($month) -anchor nw]
		set count $i
		update idletasks
		}
	$map raise $tag(Jan)
	draw_plot_axes
	$image_list selection set 0
	$image_list activate 0
	set mark_x 0
	set mark_y 0
	# .mbar.file.menu entryconfigure 0 -state disabled
	}

proc load_south {} {
	global label_list month_label
	global filename im tag ok count image_list map hemisphere
	global image_label dataset_name
	global mark_x mark_y map_width map_height

	stop_cycling

	set map_width 316
	set map_height 332
	$map configure -width $map_width -height $map_height

	set hemisphere S
	set_pole S
	set dataset_name "Southern Ocean Monthly Average\nSea-Ice Concentration"
	$image_label configure -text "$dataset_name\nPick a location on the map"

	foreach d {data/s ../data/s s} {
		if {[file exists $d] && [file isdirectory $d]} {
			break
			} \
		else {
			set d {}
			}
		}

	set label_list $month_label
	$image_list delete 0 end
	set i 0
	foreach month $label_list {
		$image_list insert end "$month"

		incr i
		set name [expr $i / 10]
		append name [expr $i % 10]
		append name s.gif
		set filename($month) $name
		# puts stdout $filename($month)

		set im($month) [image create photo -format gif -file $d/$filename($month)]
		set tag($month) [$map create image 0 0 -image $im($month) -anchor nw]
		set count $i
		update idletasks
		}
	$map raise $tag(Jan)
	draw_plot_axes
	$image_list selection set 0
	$image_list activate 0
	set mark_x 0
	set mark_y 0
	# .mbar.file.menu entryconfigure 0 -state disabled
	}

proc file_quit {} {
	exit
	}

#------------------------------------------------------------------------

proc edit_cut {} {}
proc edit_copy {} {}
proc edit_paste {} {}
proc edit_clear {} {}

#------------------------------------------------------------------------

proc show_map {name} {
	global map tag
	global show_grid gridline gridline_count
	global marker show_marker

	$map raise $tag($name)
	if {$show_grid} {
		for {set k 0} {$k < $gridline_count} {incr k} {
			$map raise $gridline($k)
			}
		}
	if {$show_marker} {
		$map raise $marker
		}
	}

proc set_current {w} {
	global tag map
	set this [$w index active]
	set name [$w get $this]
	show_map $name
	}

proc stop_cycling {} {
	global ok
	set ok 0
	}

proc cycle {} {
	global ok tag image_list map label_list;

	set ok 1
	while {1} {
		set i 0
		foreach month $label_list {
			$map raise $tag($month)
			$image_list selection clear 0 end
			$image_list selection set $i
			$image_list activate $i
			incr i
			update
			if {[expr $ok == 0]} {
				return
				}
			}
		}
	}

proc view_prev {} {
	global count image_list
	set this [$image_list index active]
	incr this -1
	if {$this < 0} {set this [expr $count - 1]}
	set name [$image_list get $this]
	show_map $name
	$image_list selection clear 0 end
	$image_list selection set $this
	$image_list activate $this
	}

proc view_next {} {
	global count image_list
	set this [$image_list index active]
	incr this
	if {$this >= $count} {set this 0}
	set name [$image_list get $this]
	show_map $name
	$image_list selection clear 0 end
	$image_list selection set $this
	$image_list activate $this
	}

proc view_first {} {
	global image_list
	set this 0
	set name [$image_list get $this]
	show_map $name
	$image_list selection clear 0 end
	$image_list selection set $this
	$image_list activate $this
	}

proc view_last {} {
	global image_list count
	set this [expr $count - 1]
	set name [$image_list get $this]
	show_map $name
	$image_list selection clear 0 end
	$image_list selection set $this
	$image_list activate $this
	}

proc view_grid {} {
	global map hemisphere
	global show_grid gridline gridline_count
	if {$gridline_count > 0} {
		if {$show_grid} {
			set show_grid 0
			for {set k 0} {$k < $gridline_count} {incr k} {
				$map lower $gridline($k)
				}
			.mbar.view.menu entryconfigure 5 -label "Show Grid"
			} \
		else {
			set show_grid 1
			for {set k 0} {$k < $gridline_count} {incr k} {
				$map raise $gridline($k)
				}
			.mbar.view.menu entryconfigure 5 -label "Hide Grid"
			}
		} \
	else {
		if {[string compare $hemisphere N] == 0} {
			set sign 1
			} \
		else {
			set sign -1
			}
		set k 0

		# Longitude lines

		for {set i 0} {$i < 360} {incr i 30} {
			set q [xy_of $i [expr $sign * 30]]
			set px [lindex $q 0]
			set py [lindex $q 1]
			set q [xy_of $i [expr $sign * 89]]
			set qx [lindex $q 0]
			set qy [lindex $q 1]
			set gridline($k) [$map create line $px $py $qx $qy -fill red]
			incr k
			}

		# Latitude of the Arctic or Antarctic circle

		for {set i 0} {$i < 360} {incr i 10} {
			set q [xy_of $i [expr $sign * 66.7]]
			set px [lindex $q 0]
			set py [lindex $q 1]
			set q [xy_of [expr $i + 5] [expr $sign * 66.7]]
			set qx [lindex $q 0]
			set qy [lindex $q 1]
			set gridline($k) [$map create line $px $py $qx $qy -fill red]
			incr k
			}

		# 10-degree lines of latitude

		set q [xy_of 0 90]
		set px [lindex $q 0]
		set py [lindex $q 1]
		for {set i 8} {$i > 3} {incr i -1} {
			set q [xy_of 0 [expr $sign * $i * 10]]
			set qx [lindex $q 0]
			set qy [lindex $q 1]
			set wx [expr $px - $qx]
			set wy [expr $py - $qy]
			set w [expr sqrt([expr $wx * $wx + $wy * $wy])]
			set h $w
			set gridline($k) [$map create oval [expr $px - $w] [expr $py - $h] [expr $px + $w] [expr $py + $h] -outline red]
			incr k
			}
		set gridline_count $k
		set show_grid 1
		.mbar.view.menu entryconfigure 5 -label "Hide Grid"
		}
	}

#------------------------------------------------------------------------
# Create a scrolled text widget
# Modified from Brent Welch's book

proc ScrolledText { f width height } {
	frame $f
	# The setgrid setting allows the window to be resized.
	text $f.text -width $width -height $height \
		-setgrid true -wrap none -padx 2 \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.yscroll -orient vertical -width 10 -command [list $f.text yview]
	pack $f.yscroll -side right -fill y
	# The fill and expand are needed when resizing.
	pack $f.text -side left -fill both -expand true
	pack $f -side top -fill both -expand true
	return $f.text
	}

#------------------------------------------------------------------------

set legend_file "legend.gif"
set metadata_file "metadata.met"

proc help_metadata {} {
	global .meta metadata_file

	if {[winfo exists .meta] == 0} {

		foreach d {doc ../doc .} {
			if {[file exists $d/$metadata_file]} {
				break
				} \
			else {
				set d {}
				}
			}

		if {[catch "open $d/$metadata_file r" fp] == 0} {
			set text_metadata [read $fp]
			close $fp
			} \
		else {
			set text_metadata "The metadata file $metadata_file could not be found in [pwd]."
			}

		toplevel .meta

		ScrolledText .meta.text 80 25
		.meta.text.text configure -background gray75
		.meta.text.text configure -font {Terminal 10}
		.meta.text.text configure -selectbackground navy -selectforeground white
		.meta.text.text configure -exportselection false
		.meta.text.text configure -tabs {32}
		.meta.text.text configure -wrap none
		.meta.text.text insert end "$text_metadata"
		pack .meta.text -fill both -expand true

		wm title .meta "Documentation"
		} \
	else {
		wm deiconify .meta
		}
	}

proc help_legend {} {
	global .legend legend_file

	if {[winfo exists .legend] == 0} {
		toplevel .legend

		foreach d {doc ../doc .} {
			if {[file exists $d/$legend_file]} {
				break
				} \
			else {
				set d {}
				}
			}

		set legend_image [image create photo -file $d/$legend_file]
		if {[string length $legend_image] > 0} {
			label .legend.label -image $legend_image
			} \
		else {
			label .legend.label -text "The legend image $legend_file could not be found."
			}

		pack .legend.label
		wm title .legend "Legend"
		} \
	else {
		wm deiconify .legend
		}
	}

proc help_version {} {
	tk_messageBox -type ok -title "About see_ice" -message "see_ice (Tcl/Tk version 1.2)\n\nPeter N. Schweitzer\nU.S. Geological Survey\npschweitzer@usgs.gov"
	}

#------------------------------------------------------------------------

proc map_position {x y} {
	global ice im LAND_CODE MISSING_CODE label_list
	global image_list image_label value_list marker show_marker map
	global hemisphere dataset_name
	global mark_x mark_y

	# Save the current pixel position in global variables

	set mark_x $x
	set mark_y $y

	# Describe the current map location

	set geo_coord [geo_of $x $y]
	set longitude [lindex $geo_coord 0]
	if {$longitude > 180} {
		set longitude [expr 360 - $longitude]
		set long_letter W
		} \
	else {
		set long_letter E
		}
	set latitude  [lindex $geo_coord 1]
	$image_label configure -text "$dataset_name\nLocation: $latitude\xB0$hemisphere, $longitude\xB0$long_letter"

	# Show the ice concentration in each layer at this location

	$value_list delete 0 end
	foreach month $label_list {
		set p [$im($month) get $x $y]
		set r [lindex $p 0]
		set g [lindex $p 1]
		set b [lindex $p 2]
		set v [ice_of_color $r $g $b]
		set ice($month) $v
		if {$v == $LAND_CODE || $v == $MISSING_CODE} {
			$value_list insert end "--"
			} \
		else {
			$value_list insert end "$v"
			}
		# puts stdout "$ice($month) $month"
		}
	set this [$image_list index active]
	set name [$image_list get $this]
	set color white
	if {$ice($name) > 40} {
		set color black
		}
	if {[info exists marker]} {
		$map delete $marker
		}
	set marker [$map create rect [expr $x - 3] [expr $y - 3] [expr $x + 3] [expr $y + 3] -outline $color]
	set show_marker 1
	draw_plot_data
	}

proc nudge {direction} {
	global mark_x mark_y map_height map_width
	switch $direction {
		up {
			if {$mark_y > 0} {
				map_position $mark_x [expr $mark_y - 1]
				}
			}
		down {
			if {$mark_y < $map_height} {
				map_position $mark_x [expr $mark_y + 1]
				}
			}
		left {
			if {$mark_x > 0} {
				map_position [expr $mark_x - 1] $mark_y
				}
			}
		right {
			if {$mark_x < $map_width} {
				map_position [expr $mark_x + 1] $mark_y
				}
			}
		upleft {
			if {$mark_y > 0} {
				set y [expr $mark_y - 1]
				}
			if {$mark_x > 0} {
				set x [expr $mark_x - 1]
				}
			map_position $x $y
			}
		upright {
			if {$mark_y > 0} {
				set y [expr $mark_y - 1]
				}
			if {$mark_x < $map_width} {
				set x [expr $mark_x + 1]
				}
			map_position $x $y
			}
		downleft {
			if {$mark_y < $map_height} {
				set y [expr $mark_y + 1]
				}
			if {$mark_x > 0} {
				set x [expr $mark_x - 1]
				}
			map_position $x $y
			}
		downright {
			if {$mark_y < $map_height} {
				set y [expr $mark_y + 1]
				}
			if {$mark_x < $map_width} {
				set x [expr $mark_x + 1]
				}
			map_position $x $y
			}
		default {
			}
		}
	}

set axis_tags {}

proc draw_plot_axes {} {
	global upa upb upc upd
	global count plot
	global label_list
	global axis_tags

	set p_height [winfo height $plot]
	set p_width [winfo width $plot]
	set font_width [font measure fixed "0"]
	set font_height [font metrics fixed -linespace]

	if {$p_height < [expr 4 * $font_height]} {
		return;
		}
	if {$p_width  < [expr 21 * $font_width]} {
		return;
		}

	if {[llength $axis_tags] > 0} {
		foreach t $axis_tags {
			$plot delete $t
			}
		set axis_tags {}
		}

	#---------------------------------------------------------------------
	# Calculate the transformation for plotting the data.
	#---------------------------------------------------------------------

	set p1x [expr 7 * $font_width]
	set p1y [expr $p_height - 2 * $font_height]
	set p2x [expr $p_width - 2 * $font_width]
	set p2y $font_height

	set u1x -0.5
	set u1y  0.0
	set u2x [expr -0.5 + $count]
	set u2y 100.0

	set upa [expr [expr $p2x - $p1x] / [expr $u2x - $u1x]]
	set upb [expr $p1x - $u1x * $upa]
	set upc [expr [expr $p2y - $p1y] / [expr $u2y - $u1y]]
	set upd [expr $p1y - $u1y * $upc]

	#---------------------------------------------------------------------
	# Axis edges (lower left)
	#---------------------------------------------------------------------

	lappend axis_tags [$plot create line $p1x $p2y $p1x $p1y]
	lappend axis_tags [$plot create line $p1x $p1y $p2x $p1y]

	#---------------------------------------------------------------------
	# X axis ticks and labels
	#---------------------------------------------------------------------

	set short_labels 0
	if {$p_width < [expr 48 * $font_width]} {
		set short_labels 1
		}

	set i 0
	foreach month $label_list {
		set px [expr $upa * $i + $upb]
		set py $p1y
		lappend axis_tags [$plot create line $px $py $px [expr $py + 4]]

		if {$short_labels} {
			set label [string index "$month" 0]
			} \
		else {
			set label "$month"
			}
		set py [expr $py + 5]
		lappend axis_tags [$plot create text $px $py -anchor n -justify center -text "$label"]

		incr i
		}

	#---------------------------------------------------------------------
	# Y axis ticks and labels
	#---------------------------------------------------------------------

	for {set i 0} {$i <= 100} {incr i 20} {
		set px $p1x
		set py [expr $upc * $i + $upd]
		lappend axis_tags [$plot create line $px $py [expr $px - 4] $py]

		set px [expr $px - 5]
		set py [expr $py + 2]
		lappend axis_tags [$plot create text $px $py -anchor e -justify center -text "$i%"]
		}
	}

proc draw_plot_data {} {
	global upa upb upc upd
	global plot ice MISSING_CODE LAND_CODE
	global segment symbol
	global label_list

	foreach month $label_list {
		if {[info exists symbol($month)]} {
			$plot delete [list $symbol($month)]
			}
		if {[info exists segment($month)]} {
			$plot delete [list $segment($month)]
			}
		}

	set x0 0
	set y0 0

	set i 0
	foreach month $label_list {
		set v $ice($month)
		if {$v == $MISSING_CODE || $v == $LAND_CODE} {
			} \
		else {
			set px [expr $upa * $i + $upb]
			set py [expr $upc * $v + $upd]
			set symbol($month) [$plot create rect [expr $px - 2] [expr $py - 2] [expr $px + 2] [expr $py + 2] -fill black]
			if {$x0 > 0 && $y0 > 0} {
				set segment($month) [$plot create line $x0 $y0 $px $py]
				}
			set x0 $px
			set y0 $py
			}
		incr i
		}
	}

proc plot_press {x} {
	global upa upb upc upd
	global count image_list

	set this [expr round([expr [expr $x - $upb] / $upa])]
	if {$this >= 0 && $this < $count} {
		set name [$image_list get $this]
		show_map $name
		$image_list selection clear 0 end
		$image_list selection set $this
		$image_list activate $this
		}
	}

#------------------------------------------------------------------------

proc query_user {} {
	global hemisphere

	toplevel .query

	label .query.title -text "Monthly Average Polar Sea-Ice Concentration" -font {Helvetica 18 bold}
	label .query.label -text "Select a set of images:"
	radiobutton .query.n -variable hemisphere -value "N" -text "Arctic Ocean and adjoining seas"
	radiobutton .query.s -variable hemisphere -value "S" -text "Southern Ocean and Antarctica"

	grid .query.title -sticky w
	grid .query.label -sticky w
	grid .query.n     -sticky w -padx 16
	grid .query.s     -sticky w -padx 16
	button .query.display -text "Display" -command {set done 1}
	button .query.cancel  -text "Cancel" -command {set hemisphere ""; set done 1}
	grid .query.display -row 4 -column 0 -sticky w -padx 4 -pady 4
	grid .query.cancel -row 4 -column 1 -sticky e -padx 4 -pady 4

	set old [focus]
	focus .query
	catch {grab .query}
	tkwait variable done
	catch {grab release .query}
	focus $old
	destroy .query
	}

#------------------------------------------------------------------------

frame .mbar -relief flat -bd 3
.mbar config -background navy
pack .mbar -side top -expand false -fill x
menubutton .mbar.file -text "File" -bg navy -fg white -menu .mbar.file.menu
menubutton .mbar.edit -text "Edit" -bg navy -fg white -menu .mbar.edit.menu
menubutton .mbar.view -text "View" -bg navy -fg white -menu .mbar.view.menu
menubutton .mbar.help -text "Help" -bg navy -fg white -menu .mbar.help.menu
pack .mbar.file .mbar.view .mbar.help -side left

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Constitute menus
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

menu .mbar.file.menu -bg blue -fg white -tearoff false
	# .mbar.file.menu add cascade -label "Open"       -menu {.mbar.file.menu.load}
	.mbar.file.menu add command -label "Quit"       -command {file_quit}

#menu .mbar.file.menu.load -bg blue -fg white -tearoff false
#	.mbar.file.menu.load add command -label "North" -command {load_north}
#	.mbar.file.menu.load add command -label "South" -command {load_south}

#menu .mbar.edit.menu -bg blue -fg white -tearoff false
#	.mbar.edit.menu add command -label "Cut"        -command {edit_cut}
#	.mbar.edit.menu add command -label "Copy"       -command {edit_copy}
#	.mbar.edit.menu add command -label "Paste"      -command {edit_paste}
#	.mbar.edit.menu add command -label "Clear"      -command {edit_clear}

menu .mbar.view.menu -bg blue -fg white -tearoff false
	.mbar.view.menu add command -label "Prev"       -command {view_prev}
	.mbar.view.menu add command -label "Next"       -command {view_next}
	.mbar.view.menu add command -label "First"      -command {view_first}
	.mbar.view.menu add command -label "Last"       -command {view_last}
	.mbar.view.menu add command -label "Cycle"      -command {cycle}
	.mbar.view.menu add command -label "Show Grid"  -command {view_grid}

menu .mbar.help.menu -bg blue -fg white -tearoff false
	.mbar.help.menu add command -label "Legend"     -command {help_legend}
	.mbar.help.menu add command -label "Metadata"   -command {help_metadata}
	.mbar.help.menu add command -label "Version"    -command {help_version}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create map, list, and plot widgets
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

frame .w
pack .w -side top -expand true -fill both

set default_background [.w cget -background]

set map .w.map
canvas $map -width 128 -height 128 -bd 0
bind $map <Button-1> {stop_cycling; map_position %x %y}

frame .w.legend
pack $map -side left -padx 8 -pady 8 -expand false
pack .w.legend -side left -padx 8 -pady 8 -expand true -fill both

#toplevel .p
set plot .w.legend.plot
canvas $plot -width 360 -height 180
pack $plot -side bottom -fill both -expand true
bind $plot <Button-1> {stop_cycling; plot_press %x}
bind $plot <Configure> {
	global count mark_x mark_y
	if {$count > 1} {
		draw_plot_axes
		map_position $mark_x $mark_y
		}
	}

frame .w.legend.layer -width 256
pack .w.legend.layer -side top -padx 8 -pady 8 -expand false

set image_label .w.legend.layer.label
label $image_label -text "" -font {Helvetica 14}
pack $image_label -side top -fill x -expand true

set image_list .w.legend.layer.list
listbox $image_list -height 12 -selectmode single -exportselection false -relief flat
$image_list configure -selectbackground navy -selectforeground white -background $default_background
bind $image_list <ButtonPress-1> {stop_cycling; %W activate @%x,%y; set_current %W}

set value_list .w.legend.layer.value
listbox $value_list -height 12 -width 5 -selectmode single -exportselection false -relief flat
$value_list configure -background $default_background
bind $value_list <ButtonPress-1> {stop_cycling; %W selection clear 0 end}
bind $value_list <ButtonRelease-1> {%W selection clear 0 end}

pack $image_list -side left -expand false
pack $value_list -side right -expand false

for {set i 1} {$i <= 12} {incr i} {
	$value_list insert end "--"
	}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

event add <<quit>> <Control-Key-q>
bind . <<quit>> {file_quit}

event add <<escape>> <Key-Escape>
bind . <<escape>> {stop_cycling}

event add <<next>> <Key-Next>
bind . <<next>> {view_next}

event add <<prev>> <Key-Prior>
bind . <<prev>> {view_prev}

event add <<first>> <Key-Home>
bind . <<first>> {view_first}

event add <<last>> <Key-End>
bind . <<last>> {view_last}

event add <<nudge_up>>     <Key-Up>    <Key-KP_8>
event add <<nudge_down>>   <Key-Down>  <Key-KP_2>
event add <<nudge_left>>   <Key-Left>  <Key-KP_4>
event add <<nudge_right>>  <Key-Right> <Key-KP_6>
event add <<nudge_upright>>            <Key-KP_9>
event add <<nudge_downright>>          <Key-KP_3>
event add <<nudge_upleft>>             <Key-KP_7>
event add <<nudge_downleft>>           <Key-KP_1>

event add <<grid>> <Key-g> <Key-G>
bind . <<grid>> {view_grid}

bind . <<nudge_up>>    {nudge up}
bind . <<nudge_down>>  {nudge down}
bind . <<nudge_left>>  {nudge left}
bind . <<nudge_right>> {nudge right}

bind . <<nudge_upleft>>  {nudge upleft}
bind . <<nudge_upright>> {nudge upright}
bind . <<nudge_downleft>>  {nudge downleft}
bind . <<nudge_downright>> {nudge downright}

wm iconify .

query_user

if {[string compare $hemisphere N] == 0} {
	load_north
	} \
else {
	if {[string compare $hemisphere S] == 0} {
		load_south
		}
	}

wm deiconify .

#========================================================================
